@@ -8,7 +8,7 @@ from cameractrls import version, ghurl
 gi.require_version('Gtk', '4.0')
 from gi.repository import Gtk, Gio, GLib, Pango, Gdk
 
-logging.getLogger().setLevel(logging.INFO)
+#logging.getLogger().setLevel(logging.INFO)
 
 class CameraCtrlsWindow(Gtk.ApplicationWindow):
     def __init__(self, *args, **kwargs):
@@ -64,9 +64,11 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
             margin-bottom: -7px;
         }
         '''
-        # XXX: remove workaround when merged
+        if (Gtk.get_major_version(), Gtk.get_minor_version()) >= (4, 12):
+            css_provider.load_from_string(css)
+        # XXX: remove workaround when merged and pygobject 3.46 is widespread
         # https://gitlab.gnome.org/GNOME/pygobject/-/merge_requests/231/
-        if (Gtk.get_major_version(), Gtk.get_minor_version()) >= (4, 9):
+        elif (Gtk.get_major_version(), Gtk.get_minor_version()) >= (4, 9):
             css_provider.load_from_data(css, -1)
         else:
             css_provider.load_from_data(css.encode())
@@ -76,9 +78,25 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
             Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
         )
 
+        try:
+            self.desktop_portal = Gio.DBusProxy.new_for_bus_sync(
+                Gio.BusType.SESSION, Gio.DBusProxyFlags.NONE, None,
+                'org.freedesktop.portal.Desktop', '/org/freedesktop/portal/desktop',
+                'org.freedesktop.portal.Settings', None)
+            prefer_dark = self.desktop_portal.Read('(ss)', 'org.freedesktop.appearance', 'color-scheme') == 1
+            self.get_settings().set_property('gtk-application-prefer-dark-theme', prefer_dark)
+            self.desktop_portal.connect('g-signal', lambda p, sender, signal, params:
+                signal == 'SettingChanged' and
+                params[0] == 'org.freedesktop.appearance' and
+                params[1] == 'color-scheme' and
+                self.get_settings().set_property('gtk-application-prefer-dark-theme', params[2] == 1)
+            )
+        except Exception as e:
+            logging.warning(f'desktop_portal failed: {e}')
+
         about_button = Gtk.Button(
             action_name='app.about',
-            icon_name='info-symbolic',
+            icon_name='open-menu-symbolic',
             has_frame=False,
             tooltip_text='About',
         )
@@ -243,8 +261,6 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         if self.frame:
             self.grid.remove(self.frame)
             self.frame = None
-            # forget old size
-            self.set_default_size(-1, -1)
 
         if self.device is None:
             return
@@ -257,7 +273,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         stack = Gtk.Stack(transition_type=Gtk.StackTransitionType.SLIDE_LEFT_RIGHT, transition_duration=500)
         stack_sw = Gtk.StackSwitcher(stack=stack, hexpand=True, halign=Gtk.Align.CENTER)
         stack_box.append(stack_sw)
-        stack_box.append(stack)
+        scrolledwindow = Gtk.ScrolledWindow(child=stack, propagate_natural_height=True, hscrollbar_policy=Gtk.PolicyType.NEVER, vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
+        stack_box.append(scrolledwindow)
 
         footer = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin_top=10)
         stack_box.append(footer)
@@ -388,6 +405,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
                         for m in filtered_menu:
                             b = Gtk.Button(label=m.name, valign=Gtk.Align.CENTER)
                             b.connect('clicked', lambda e, c=c, m=m: self.update_ctrl(c, m.text_id))
+                            if c.child_tooltip:
+                                b.set_tooltip_markup(c.child_tooltip)
                             if m.lp_text_id is not None:
                                 lp = Gtk.GestureLongPress(propagation_phase=Gtk.PropagationPhase.CAPTURE)
                                 lp.connect('pressed', lambda lp, x, y, c=c, m=m, b=b: [
@@ -458,6 +477,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
                             c.gui_default_btn = refresh
 
         self.update_ctrls_state()
+        _, natsize = self.get_preferred_size()
+        self.set_default_size(natsize.width, natsize.height)
 
     def close_notify(self):
         self._revealer.set_reveal_child(False)
@@ -483,10 +504,11 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
             if errs:
                 self.notify('\n'.join(errs))
                 GLib.idle_add(self.update_ctrl_value, ctrl),
+                return
 
         self.update_ctrls_state()
         if ctrl.reopener:
-            self.reopen_device()
+            GLib.idle_add(self.reopen_device)
 
     def update_ctrls_state(self):
         for c in self.camera.get_ctrls():
@@ -654,7 +676,7 @@ class CameraCtrlsApp(Gtk.Application):
         about.set_authors(['Gergo Koteles <soyer@irl.hu>'])
         about.set_artists(['Jorge Toledo https://lacajita.es'])
         about.set_copyright('Copyright © 2022 - 2024 Gergo Koteles')
-        about.set_license_type(Gtk.License.MIT_X11)
+        about.set_license_type(Gtk.License.GPL_3_0)
         about.set_logo_icon_name('hu.irl.cameractrls')
         about.set_website(ghurl)
         about.set_website_label('GitHub')
